#include "lib_indicator.h"
#include "lib_dmx_in.h"
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>


	
volatile uint8_t   gFlags;
		 uint16_t  PatMask= 1;
		 uint8_t   IndCnt= 2;
		 uint16_t  Pump= 0;


enum {TICK};

#define IND_TIME	(1)


//PD5 = manual trigger

//PD3 = pump output
//PA0 = heater on/off (future)
//PA1 = Fan output (future)
//PA7 = manual indicator LED (future)


const uint16_t PumpTable[] PROGMEM =	{
		0b0000000000000000,
		0b0000000000000001,
		0b0000000100000001,
		0b0000100001000001,
		0b0001000100010001,
		0b0101010101010101,
		0b1110111011101110,
	//	0b1111111011111110,
		0b1111111111111111
		};

// *************** Initialization ****************
void init_system(void)
{
wdt_reset();
wdt_enable(WDTO_500MS);											//enable watchdog (Timeout= 0.5s)
wdt_reset();

DDRA=  0xFF;													//high outputs
PORTA= 0xFE;

DDRD=  0b10001110;												//DMX, spare, red LED on (use zc as pump output)
PORTD= 0b01110010;

OCR1A  = 1250;													//set system clock to 25 Hz
TCCR1A = 0;
TCCR1B = (1<<CS12)|(1<<WGM12);									//set T1 @clock/256, CTC mode 
TIMSK  = (1<<OCIE1A);											//enable compare and overflow interrupts (u-step, system)

IndFlags = (1<<COLD);

init_ind();
init_DMX_RX();
get_dips();
}



int main(void)

{
cli();
init_system();
sei();
for(;;)
	{
	if (gFlags &(1<<TICK))								//call with 25Hz
		{
		gFlags &= ~(1<<TICK);
		wdt_reset();
		if (IndCnt-- == 0)
			{
			IndCnt= IND_TIME;							//indicate with 12.5 Hz
			get_dips();
			if (PINE &(1<<PE1)) UCSRB= (1<<RXEN)|(1<<RXCIE);
			else
				{
				UCSRB= 0;								//manual mode
				IndFlags |= (1<<VALID_DMX);
				DmxRxField[0]= 0xFF;					//enable
				if (PINE &(1<<PE2)) DmxRxField[2]= 0;	//fan
				else				DmxRxField[2]= 0xFF;
				if (!(PINC &(1<<PC7)) || !(PIND &(1<<PD5))) DmxRxField[1]= ((0xFF -PINC)<<1)&0xFF; //pump
				else DmxRxField[1]= 0;
				}
			indicate();
			}
		
		if (IndFlags &(1<<DMX_ERR)) Pump= 0;			//stop pumping if too cold or err
		else				  	    Pump= pgm_read_word(&PumpTable[DmxRxField[0]>>5]);
		if (DmxRxField[1]>127)	 PORTA |=  (1<<PA1);	//enable fan
		else					 PORTA &= ~(1<<PA1);
		if (DmxRxField[2]>127)	 PORTA |=  (1<<PA0);	//enable whole fogger
		else					 PORTA &= ~(1<<PA0);
		}
	}
}



// *************** ISRs ****************

ISR (TIMER1_COMPA_vect)									
{
gFlags |= (1<<TICK);

PatMask= (PatMask>>1);
if (PatMask == 0)PatMask= 0x8000; 

if (Pump &PatMask)
	{
	PORTD |= (1<<PD3);
	}
else
	{
	PORTD &= ~(1<<PD3);
	}
}
